/*
 * $Id: AssistPlugin.java,v 1.20 2005/01/13 14:50:17 dallmeier Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.assist;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Vector;

import org.eclipse.compare.ResourceNode;
import org.eclipse.compare.structuremergeviewer.DocumentRangeNode;
import org.eclipse.compare.structuremergeviewer.IStructureComparator;
import org.eclipse.compare.structuremergeviewer.IStructureCreator;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.team.internal.ccvs.core.CVSException;
import org.eclipse.team.internal.ccvs.core.resources.CVSWorkspaceRoot;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorRegistry;
import org.eclipse.ui.IStartup;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.eclipse.ui.texteditor.ITextEditor;
import org.osgi.framework.BundleContext;
import org.softevo.rose.assist.mining.preferences.IRosePreferenceConstants;
import org.softevo.rose.compare.ModificationItem;
import org.softevo.rose.compare.RoseComparePlugin;

/**
 * The main class of the plugin that contains the
 * views for suggestions. <BR>
 */
public class AssistPlugin extends AbstractUIPlugin implements IResourceChangeListener, IStartup
{
  
  //The shared instance.
  private static AssistPlugin plugin;
  
  //Resource bundle.
  private ResourceBundle resourceBundle;
  
  private static final String ID = "org.softevo.rose.assist";

  private boolean IS_ENABLED = true;
  private boolean IS_PREPROCESSING = false;
  
  /** Listeners for changes in the list of changed entities. <BR>
   */
  private Vector changeListeners = null;
  
  /** The list of changed entities. <BR>
   */
  private HashMap changedEntities = null;
  
  /**
   * The constructor.
   */
  public AssistPlugin() {
    super();
    plugin = this;
    try {
      resourceBundle = ResourceBundle.getBundle("org.softevo.rose.assist.AssistPluginResources");
    } catch (MissingResourceException x) {
      resourceBundle = null;
    }
    changeListeners = new Vector();
    changedEntities = new HashMap();
  }
  
  /**
   * This method is called upon plug-in activation
   */
  public void start(BundleContext context) throws Exception {
    super.start(context);
    ResourcesPlugin.getWorkspace().addResourceChangeListener(this,
                                                             IResourceChangeEvent.PRE_CLOSE
                                                             | IResourceChangeEvent.PRE_DELETE
                                                             | IResourceChangeEvent.POST_CHANGE);
    IS_ENABLED = getPreferenceStore().getBoolean(IRosePreferenceConstants.ROSE_IS_ENABLED);
  }
  
  /**
   * This method is called when the plug-in is stopped
   */
  public void stop(BundleContext context) throws Exception {
    super.stop(context);
  }
  
  /**
   * Returns the shared instance.
   */
  public static AssistPlugin getDefault() {
    return plugin;
  }
  
  /**
   * Returns the string from the plugin's resource bundle,
   * or 'key' if not found.
   */
  public static String getResourceString(String key) {
    ResourceBundle bundle = AssistPlugin.getDefault().getResourceBundle();
    try {
      return (bundle != null) ? bundle.getString(key) : key;
    } catch (MissingResourceException e) {
      return key;
    }
  }
  
  /**
   * Returns the plugin's resource bundle,
   */
  public ResourceBundle getResourceBundle() {
    return resourceBundle;
  }
  
  /**
     * This method is called whenever a resource in the workspace has changed.
     * <BR>
     * If the change occured in a file, an attempt is made to find the changed
     * entities.
     * 
     * @param event
     *            the event describing the change
     */
    public void resourceChanged(IResourceChangeEvent event) {
        
        if (!IS_ENABLED) return;
        if (IS_PREPROCESSING) return;
        
        IResourceDeltaVisitor projectProcessor;
        final BooleanBox changed = new BooleanBox(false);
        final Vector changedProjects = new Vector();
        
        if (event.getType() == IResourceChangeEvent.POST_CHANGE) {
            // Process delta until an IProject is found
            projectProcessor = new IResourceDeltaVisitor() {
                public boolean visit(IResourceDelta delta) throws CoreException {
                    IResource resource = delta.getResource();
                    if (resource instanceof IProject) {
                        IProject project = (IProject) resource;
                        // Skip if no database provider is specified. FIXME (klz) Why fails the following line???
                        // if (DbPlugin.getDefault().getProviderForProject(project)==null) return false;
                        // Execute DeltaProcessor for each project individually
                        DeltaProcessor processor = new DeltaProcessor();
                        delta.accept(processor);
                        if (processor.hasChangedEntities()) {
                            changedProjects.add(project);
                            // Get (and create if necessary) the changedEntities Vector for a project
                            if (!changedEntities.containsKey(project)) {
                                changedEntities.put(project, new Vector());
                            }
                            Vector changedEntitiesForProject = (Vector) changedEntities.get(project);
                            // Insert the changed entities
                            Vector deltaChanges = processor.getChangedEntities();
                            Iterator iter = deltaChanges.iterator();
                            while (iter.hasNext()) {
                                Object object = iter.next();
                                if (!changedEntitiesForProject.contains(object)) {
                                    changedEntitiesForProject.add(object);
                                    changed.b = true;
                                } else {
                                    int pos = changedEntitiesForProject.indexOf(object);
                                    ModificationItem item = (ModificationItem) object;
                                    ModificationItem oldItem = (ModificationItem) changedEntitiesForProject.get(pos);
                                    if (oldItem.getType() == ModificationItem.ADDED && item.getType() == ModificationItem.DELETED) {
                                        changedEntitiesForProject.remove(pos);
                                        changed.b = true;
                                    } else if (oldItem.getType() == ModificationItem.DELETED && item.getType() == ModificationItem.ADDED) {
                                        changedEntitiesForProject.remove(pos);
                                        changed.b = true;
                                    } else if (oldItem.getType() == ModificationItem.CHANGED && item.getType() == ModificationItem.DELETED) {
                                        oldItem.changeType(ModificationItem.DELETED);
                                        changed.b = true;
                                    }
                                }
                            }
                        }
                        return false;
                    }
                    return true;
                }
            };
            try {
                event.getDelta().accept(projectProcessor);
            } catch (CoreException coreException) {
                //System.out.println("An exception occured processing a resource delta.");
                //coreException.printStackTrace();
                AssistPlugin.logException(coreException, IStatus.WARNING);
            }
            if (changed.b) {
                fireChangeListeners(changedProjects);
            }
        }
    }
  
  /** Notifies all registered listeners of a change
   *  in the list of changed entities. <BR>
   */
  private void fireChangeListeners(Vector changedProjects) {
    for (int counter = 0; counter < changeListeners.size(); counter++) {
      ((IChangedEntitiesListener) changeListeners.get(counter)).updateChangedEntities(changedEntities, changedProjects);
    }
  }
  
  /** Removes all elements from the list of
   *  changed entities and notifies the listeners. <BR>
   */
  public void clearChangedEntitiesList() {
    changedEntities.clear();
    fireChangeListeners(null);
  }
  
  /** Gets the list of changed entities. <BR>
   */
  public Vector getChangedEntitiesForProject(IProject project) {
    return (Vector) changedEntities.get(project);
  }
  
  /** Removes a previously registered listener
   *  for changed entitities. <BR>
   *  @param listener the listener to remove
   *  @return <code>true</code> if the listener was added before,
   *          <code>false</code> otherwise
   */
  public boolean removeChangedEntitiesListener(IChangedEntitiesListener listener) {
    return changeListeners.remove(listener);
  }
  
  /** Adds a new listener for changed entities. <BR>
   *  Listeners are added only once.
   *  @param listener the listener to be added
   */
  public void addChangedEntitiesListener(IChangedEntitiesListener listener) {
    if (!changeListeners.contains(listener)) {
      changeListeners.add(listener);
    }
  }

  private class BooleanBox {
      public boolean b = false;
      public BooleanBox() {
        //default is false
      }
      public BooleanBox(boolean b) {
          this.b = b;
      }
  }

/**
 * @param fileName
 * @param entityName
 * @param projects
 */
public void openInEditor(String fileName, String entityName, IProject[] projects) {
    
    // Find the correct project
    IProject project = null;
    String remoteName = null;
    for (int i = 0; i < projects.length; i++) {
        try {
            remoteName = CVSWorkspaceRoot.getCVSFolderFor(projects[i]).getRepositoryRelativePath();
            if (fileName.startsWith("/" + remoteName)) {
                project = projects[i];
                break;
            }
        } catch (CVSException e) {
            //ignore
        }
    }
    
	IResource res = null;
	if (project == null) {
	    res = ResourcesPlugin.getWorkspace().getRoot().findMember(fileName);
	} else {
	    res = project.findMember(fileName.substring(remoteName.length() + 2 /*two slashes*/));
	}
	if (res == null || !(res instanceof IFile)) return;

	// Open the file
	IFile file = (IFile) res; 
	IEditorRegistry editorRegistry = PlatformUI.getWorkbench().getEditorRegistry();
	IEditorDescriptor editorDesc = editorRegistry.getDefaultEditor(file.getName());
	if (editorDesc == null) {
		editorDesc = editorRegistry.getDefaultEditor("a.txt"); // FIXME
	}

	IEditorPart editorPart = null;
	if (editorDesc != null) {
		IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		try {
			editorPart = page.openEditor(new FileEditorInput(file), editorDesc.getId(), true);
		} catch (PartInitException e1) {
            AssistPlugin.logException(e1, Status.WARNING);
			return;
		}
	} else {
	    return;
	}

	// Open item in editor
	if (!entityName.equals(fileName) && editorPart instanceof ITextEditor) {
		IStructureCreator creator = RoseComparePlugin.getStructureCreator(file.getFileExtension());
		if (creator != null) {
		    IStructureComparator structure = creator.getStructure(new ResourceNode(file));
		    if (structure instanceof DocumentRangeNode) {
		        LinkedList stack = new LinkedList();
		        stack.addFirst(structure);
		        
		        while (!stack.isEmpty()) {
			        DocumentRangeNode drn = (DocumentRangeNode) stack.removeFirst();
			        if (entityName.equals(drn.getId())) {
			            int offset = drn.getRange().getOffset();
			            ((ITextEditor) editorPart).selectAndReveal(offset, 0);
			            return;
			        }
			        for (int i = 0; i < drn.getChildren().length; i++) {
			            Object o = drn.getChildren()[i];
                        if (o instanceof DocumentRangeNode) {
                            stack.addFirst(o);
                        }
                    }
		        }
		        
		    }
		}
	}
}

    protected void initializeImageRegistry(ImageRegistry registry) {
        super.initializeImageRegistry(registry);
		put(registry, "change", "icons/change.gif"); //$NON-NLS-1$//$NON-NLS-2$
		put(registry, "minus", "icons/minus.gif"); //$NON-NLS-1$//$NON-NLS-2$
		put(registry, "plus", "icons/plus.gif"); //$NON-NLS-1$//$NON-NLS-2$
		put(registry, "rose", "icons/rose.gif"); //$NON-NLS-1$//$NON-NLS-2$
		put(registry, "newlocation", "icons/newlocation_rose_wizban.gif"); //$NON-NLS-1$//$NON-NLS-2$
    }
    
	private void put(ImageRegistry registry, String key, String fileName) {
		registry.put(key, ImageDescriptor.createFromURL(Platform.find(getBundle(), new Path(fileName))));	
	}
	
	public static void debugInfo(String message) {
	    if (plugin.isDebugging())
	        System.out.println(message);
	}

	public static void logInfo(String message) {
	    if (plugin.isDebugging())
	        plugin.getLog().log(new Status(IStatus.INFO, ID, 0, message, null));
	}
	
	public static void logWarning(String message, Throwable e) {
		plugin.getLog().log(new Status(IStatus.WARNING, ID, 0, message, e));
	}

	public static void logError(String message, Throwable e) {
		plugin.getLog().log(new Status(IStatus.ERROR, ID, 0, message, e));
	}
	
	public static void logException(Throwable e, int severity) {
	    plugin.getLog().log(new Status(severity, ID, 0, e.getMessage(), e));
	} 

	public boolean isEnabled() {
	    return IS_ENABLED;
	}
	
	public void setEnabled(boolean b) {
	    if (IS_ENABLED==b) return;
	    IS_ENABLED = b;
	    getPreferenceStore().setValue(IRosePreferenceConstants.ROSE_IS_ENABLED, b);
	    clearChangedEntitiesList();
	}

	public boolean isPreprocessing() {
	    return IS_PREPROCESSING;
	}
	
	public void setPreprocessing(boolean b) {
	    IS_PREPROCESSING = b;
	}

	
    /* (non-Javadoc)
     * @see org.eclipse.ui.IStartup#earlyStartup()
     */
    public void earlyStartup() {
        // TODO Auto-generated method stub
        
    }
 }
