/*
 * $Id: DataMiningResult.java,v 1.3 2005/01/12 16:06:48 dallmeier Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.assist;

/** A default implementation of the <code>IDataMiningResult</code> interface. <BR>
 *  This class wraps the properties that describe the result of
 *  a data mining operation. <BR>
 */
public class DataMiningResult implements IDataMiningResult {
  
  /** The name of the file. <BR>
   */
  private String fileName;
  
  /** The name of the entity where changes occured. <BR>
   */
  private String entityName;
  
  /** The identifier for the resource. <BR>
   */
  private String id;
  
  /** The confidence for the result. <BR>
   */
  private double confidence = 0;
  
  /** The support. <BR>
   */
  private int support = -1;
  
  /** The action to perform. <BR>
   */
  private String action = null;
  
  /** Creates a new data mining result. <BR>
   *  @param entityName the name of the entity
   *  @param fileName the location
   *  @param id the identifier for the resource
   */
  public DataMiningResult(String entityName, String fileName, String id) {
    this.entityName = entityName;
    this.fileName = fileName;
    this.id = id;
  }
  
  public String getAction() {
    return action;
  }
  public void setAction(String action) {
    this.action = action;
  }
  public void setSupport(int support) {
    this.support = support;
  }
  public void setConfidence(double conf) {
    this.confidence = conf;
  }
  
  /* (non-Javadoc)
   * @see org.softevo.rose.assist.IDataMiningResult#getConfidence()
   */
  public double getConfidence() {
    return confidence;
  }
  
  /* (non-Javadoc)
   * @see org.softevo.rose.assist.IDataMiningResult#getSupport()
   */
  public int getSupport() {
    return support;
  }
  
  /* (non-Javadoc)
   * @see org.softevo.rose.assist.INamedObject#getFileName()
   */
  public String getFileName() {
    return fileName;
  }
  
  /* (non-Javadoc)
   * @see org.softevo.rose.assist.INamedObject#getEntityName()
   */
  public String getEntityName() {
    return entityName;
  }
  
  /* (non-Javadoc)
   * @see org.softevo.rose.assist.INamedObject#getId()
   */
  public String getId() {
    return id;
  }
  
}
