/*
 * $Id: AssociationRuleMiningPlugin.java,v 1.11 2005/02/06 20:13:16 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.assist.mining;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.TreeSet;
import java.util.Vector;

import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;
import org.softevo.rose.RosePluginException;
import org.softevo.rose.assist.AssistPlugin;
import org.softevo.rose.assist.DataMiningResult;
import org.softevo.rose.assist.IDataMiningResult;
import org.softevo.rose.assist.mining.entities.EntityTypeAddIn;
import org.softevo.rose.assist.mining.entities.EntityTypeChange;
import org.softevo.rose.assist.mining.entities.EntityTypeDelIn;
import org.softevo.rose.assist.mining.entities.IEntityType;
import org.softevo.rose.assist.mining.preferences.IRosePreferenceConstants;
import org.softevo.rose.compare.ModificationItem;
import org.softevo.rose.data.entities.db.Tables;
import org.softevo.rose.db.DatabaseProvider;
import org.softevo.util.StringUtils;

/**
 * TODO Mehrere Datenbanken, mehrere Projekte, Zuordnung unklar.
 * 
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public class AssociationRuleMiningPlugin extends AbstractMiningPlugin {

    public static final IEntityType[] ENTITY_TYPES = {
        new EntityTypeAddIn(),
        new EntityTypeDelIn(),
        new EntityTypeChange()
    };
    
    public static final int[] SUPPORT_VALUES = { 1, 2, 3, 4, 5, 6, 7, 8, 9 };
    public static final double[] CONFIDENCE_VALUES = { 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0 };
    
    private static final String TABLE_CVS_FILES = Tables.TABLE_FILES;
    private static final String TABLE_CVS_ENTITIES = Tables.PREFIX_CVS + Tables.TABLE_ENTITIES;
    private static final String TABLE_LINEITEMS = org.softevo.rose.assist.mining.db.Tables.PREFIX_CVS + 
    	org.softevo.rose.assist.mining.db.Tables.TABLE_LINEITEMS;

    private PreparedStatement entityStmt = null;
    private PreparedStatement entityAddedStmt = null;
    private Statement miningStmt = null;
    
    private int minSupport = 0;
    private double minConfidence = 0.0;
    private Vector relevantActions = new Vector();
    
    /**
     */
    public AssociationRuleMiningPlugin() {
        refreshProperties();
    }

    /**
     * @see org.softevo.rose.assist.IDataMiningAlgorithm#performMining(org.softevo.rose.compare.ModificationItem[])
     */
    public Object[] performMining(ModificationItem[] changedEntities) {
        
        // Get entitiy ids for changedEntities
        TreeSet entityIDs = new TreeSet();
        TreeSet fileNames = new TreeSet();
        for (int i = 0; i < changedEntities.length; i++) {
            String fileName = changedEntities[i].getAttachedFileName();
            String id = changedEntities[i].getID();
            try {
                switch (changedEntities[i].getType()) {
                case ModificationItem.ADDED:
                case ModificationItem.DELETED:
                    entityAddedStmt.setString(1, fileName);
	                if (entityAddedStmt.execute()) {
	                    ResultSet rset = entityAddedStmt.getResultSet();
	                    if (rset.next()) {
	                        entityIDs.add(new Integer(rset.getInt(1)));
	                        fileNames.add(fileName);
	                        while (rset.next()) {
	                            AssistPlugin.logWarning("Warning: Found additional entity " + rset.getInt(1) + " for " + id + " in " + fileName, null);
	                        }
	                    }
	                }
                    break;
                default:
                    entityStmt.setString(1, fileName);
	                entityStmt.setString(2, id);
	                if (entityStmt.execute()) {
	                    ResultSet rset = entityStmt.getResultSet();
	                    if (rset.next()) {
	                        entityIDs.add(new Integer(rset.getInt(1)));
	                        fileNames.add(fileName);
	                        while (rset.next()) {
	                            AssistPlugin.logWarning("Warning: Found additional entity " + rset.getInt(1) + " for " + id + " in " + fileName, null);
	                        }
	                    }
	                }
                    break;
                }
            } catch (SQLException e) {
                AssistPlugin.logWarning("Warning: Failed to identify " + id + " in " + fileName, e);
            }
        }

        // Perform mining
        String actionList = relevantActions.size()==0 ? "" : 
            "'" + StringUtils.join(relevantActions.iterator(), "', '") + "'";
        String miningQuery = "SELECT EntityId, Action, COUNT(*) AS Frequency "
			+ "FROM " + TABLE_LINEITEMS + " l, "
			+ "(SELECT TransactionId FROM " + TABLE_LINEITEMS + " "
				+ "WHERE EntityId IN ( " + StringUtils.join(entityIDs.iterator(), ", ") + " ) "
				+ "GROUP BY TransactionId "
				+ "HAVING COUNT(*)=" + entityIDs.size() + " "
				+ ") foo "
			+ "WHERE l.TransactionID=foo.TransactionID "
			+ "AND ACTION IN ( " + actionList + " ) " 
			+ "GROUP BY EntityId, Action "
			+ "HAVING COUNT(*)>=" + minSupport;
        String resultQuery = "SELECT m.EntityId, m.Frequency, e.EntityName, f.QualifiedFileName, m.Action "
            + "FROM ( " + miningQuery + " ) m, "
            	+ TABLE_CVS_ENTITIES + " e, " + TABLE_CVS_FILES + " f "
            + "WHERE m.EntityID=e.EntityID AND e.FileID=f.FileID " 
			+ "ORDER BY m.Frequency DESC";
        
        // Create the mining results
        AssistPlugin.debugInfo(resultQuery);
        if (entityIDs.size()==0) return new IDataMiningResult[0];
        
        Vector result = new Vector();
        try {
            ResultSet rset = miningStmt.executeQuery(resultQuery);
            int maxSupport = -1;
            while (rset.next()) {
                int support = rset.getInt(2);
                if (support > maxSupport) maxSupport = support; // This should be changed ionly once.
                if (!entityIDs.contains((new Integer(rset.getInt(1))))) {
	                String entityName = rset.getString(3);
	                String fileName = rset.getString(4);
	                double confidence = support*1.0/maxSupport;
	                if (confidence > minConfidence) {
	                        //&& (!entityName.equals(fileName) || !fileNames.contains(fileName))) {
		                DataMiningResult item = new DataMiningResult(entityName, fileName, entityName);
		                item.setConfidence(confidence);
		                item.setSupport(support);
		                item.setAction(rset.getString(5));
		                result.add(item);
	                }
                }
            }
        } catch (SQLException e) {
            AssistPlugin.logException(e, Status.WARNING);
            return new IDataMiningResult[0];
        }
        return result.toArray(new IDataMiningResult[0]);
    }

    /**
     * @see org.softevo.rose.assist.IDataMiningAlgorithm#getName()
     */
    public String getName() {
        return "Association Rule Mining";
    }

    public void close() throws RosePluginException {
        try {
            entityStmt.close();
            entityAddedStmt.close();
            miningStmt.close();
        } catch (SQLException e) {
            throw new RosePluginException(e);
        } finally {
            super.close();
        }
    }
    public void init(DatabaseProvider db) throws RosePluginException {
        super.init(db);
        
        String query = "SELECT entityID FROM " + TABLE_CVS_ENTITIES + " e, " + TABLE_CVS_FILES + " f "
        	+ "WHERE e.FileID=f.FileID AND f.QualifiedFileName=? AND e.EntityName=?";              
        try {
            entityStmt = db.getConnection().prepareStatement(query);
        } catch (SQLException e) {
            throw new RosePluginException(e);
        }
        query = "SELECT entityID FROM " + TABLE_CVS_ENTITIES + " e, " + TABLE_CVS_FILES + " f "
	    	+ "WHERE e.FileID=f.FileID AND f.QualifiedFileName=? AND e.EntityName=f.QualifiedFileName";              
	    try {
	        entityAddedStmt = db.getConnection().prepareStatement(query);
	    } catch (SQLException e) {
	        throw new RosePluginException(e);
	    }
        try {
            miningStmt = db.getConnection().createStatement();
        } catch (SQLException e) {
            throw new RosePluginException(e);
        }
    }

    
    public void refreshProperties() {
        IPreferenceStore store = AssistPlugin.getDefault().getPreferenceStore();
        if (store.getBoolean(IRosePreferenceConstants.ROSE_IS_ENABLED_MIN_SUPPORT)) {
            minSupport = store.getInt(IRosePreferenceConstants.ROSE_MIN_SUPPORT);
        } else {
            minSupport = 0;
        }
        if (store.getBoolean(IRosePreferenceConstants.ROSE_IS_ENABLED_MIN_CONFIDENCE)) {
            minConfidence = store.getDouble(IRosePreferenceConstants.ROSE_MIN_CONFIDENCE);            
        } else {
            minConfidence = 0;
        }
        relevantActions = new Vector(ENTITY_TYPES.length);
        for (int i = 0; i < ENTITY_TYPES.length; i++) {
            if (store.getBoolean(ENTITY_TYPES[i].getPreferenceID())) {
                relevantActions.add(ENTITY_TYPES[i].getName());
            }
        }
    }
    
}
