/*
 * $Id: RosePreferencePage2.java,v 1.5 2004/12/27 18:36:02 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.assist.mining.preferences;

import java.text.Collator;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;

import org.eclipse.jface.preference.IPreferencePage;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.softevo.rose.assist.AssistPlugin;
import org.softevo.rose.assist.mining.AssociationRuleMiningPlugin;
import org.softevo.rose.assist.mining.entities.IEntityType;

/**
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public class RosePreferencePage2 extends PreferencePage implements IWorkbenchPreferencePage {

    private Text descriptionText;

    private CheckboxTableViewer checkboxViewer;

    private CheckboxLabelCombo supportBox;

    private CheckboxLabelCombo confidenceBox;
    
    private String[] supportValues;
    private String[] confidenceValues;
    
    
    /**
     * @see PreferencePage#createContents(Composite)
     */
    protected Control createContents(Composite parent) {

        Font font = parent.getFont();

        Composite mainComposite = new Composite(parent, SWT.NONE);
        mainComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        mainComposite.setFont(font);

        GridLayout layout = new GridLayout();
        layout.marginWidth = 0;
        layout.marginHeight = 0;
        layout.verticalSpacing = 10;
        mainComposite.setLayout(layout);

        createFilterArea(mainComposite);
        createEntityTypesArea(mainComposite);
        createDescriptionArea(mainComposite);

        initializeValues();

        return mainComposite;
    }

    /** 
     * Creates the widgets for the description.
     * @param mainComposite
     */
    private void createFilterArea(Composite mainComposite) {

        Font mainFont = mainComposite.getFont();
        Composite textComposite = new Composite(mainComposite, SWT.NONE);
        textComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        GridLayout textLayout = new GridLayout();
        textLayout.marginWidth = 0;
        textLayout.marginHeight = 0;
        textComposite.setLayout(textLayout);
        textComposite.setFont(mainFont);

        Label descriptionLabel = new Label(textComposite, SWT.NONE | SWT.WRAP);
        descriptionLabel.setText("eROSE can filter association rules by support and confidence."); //$NON-NLS-1$
        descriptionLabel.setFont(mainFont);

        supportBox = new CheckboxLabelCombo(textComposite, "Filter by support:", true);
        confidenceBox = new CheckboxLabelCombo(textComposite, "Filter by confidence:", true);
        
        supportValues = new String[AssociationRuleMiningPlugin.SUPPORT_VALUES.length];
        for (int i = 0; i < supportValues.length; i++) {
            supportValues[i] = Integer.toString(AssociationRuleMiningPlugin.SUPPORT_VALUES[i]);
        }
        supportBox.getCombo().setItems(supportValues);

        confidenceValues = new String[AssociationRuleMiningPlugin.CONFIDENCE_VALUES.length];
        for (int i = 0; i < confidenceValues.length; i++) {
            confidenceValues[i] = Double.toString(AssociationRuleMiningPlugin.CONFIDENCE_VALUES[i]);
        }
        confidenceBox.getCombo().setItems(confidenceValues);

    }

    
    /** 
     * Creates the widgets for the list of decorators.
     * @param mainComposite
     */
    private void createEntityTypesArea(Composite mainComposite) {

        Font mainFont = mainComposite.getFont();

        GridLayout labelLayout = new GridLayout();
        labelLayout.marginWidth = 0;
        labelLayout.marginHeight = 0;
        labelLayout.verticalSpacing = 0;
        
        Composite labelComposite = new Composite(mainComposite, SWT.NONE);
        labelComposite.setLayout(labelLayout);
        Label descLabel = new Label(labelComposite, SWT.NONE);
        descLabel.setText("eROSE uses different entity types to distinguish between "); //$NON-NLS-1$
        descLabel.setFont(mainFont);
        descLabel = new Label(labelComposite, SWT.NONE);
        descLabel.setText("changes, additions, deletions, and other events."); //$NON-NLS-1$
        descLabel.setFont(mainFont);
        
        GridLayout gridLayout = new GridLayout();
        gridLayout.marginWidth = 0;
        gridLayout.marginHeight = 0;

        Composite entityTypesComposite = new Composite(mainComposite, SWT.NONE);
        entityTypesComposite.setLayoutData(new GridData(GridData.FILL_BOTH));
        entityTypesComposite.setLayout(gridLayout);
        entityTypesComposite.setFont(mainFont);

        Label entityLabel = new Label(entityTypesComposite, SWT.NONE);
        entityLabel.setText("Available entity types:"); //$NON-NLS-1$
        entityLabel.setFont(mainFont);

        // Checkbox table viewer of decorators
        checkboxViewer = CheckboxTableViewer.newCheckList(entityTypesComposite,
                SWT.SINGLE | SWT.TOP | SWT.BORDER);
        checkboxViewer.getTable().setLayoutData(new GridData(GridData.FILL_BOTH));
        checkboxViewer.getTable().setFont(entityTypesComposite.getFont());
        checkboxViewer.setLabelProvider(new LabelProvider() {
            public String getText(Object element) {
                return ((IEntityType) element).getName();
            }
        });
        checkboxViewer.getTable().setFont(mainFont);

        checkboxViewer.setContentProvider(new IStructuredContentProvider() {
            private final Comparator comparer = new Comparator() {
                private Collator collator = Collator.getInstance();

                public int compare(Object arg0, Object arg1) {
                    String s1 = ((IEntityType) arg0).getName();
                    String s2 = ((IEntityType) arg1).getName();
                    return collator.compare(s1, s2);
                }
            };

            public void dispose() {
                //Nothing to do on dispose
            }

            public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
                // empty
            }

            public Object[] getElements(Object inputElement) {
                //Make an entry for each decorator definition
                Object[] elements = (Object[]) inputElement;
                Object[] results = new Object[elements.length];
                System.arraycopy(elements, 0, results, 0, elements.length);
                Collections.sort(Arrays.asList(results), comparer);
                return results;
            }

        });

        checkboxViewer.addSelectionChangedListener(new ISelectionChangedListener() {
                    public void selectionChanged(SelectionChangedEvent event) {
                        if (event.getSelection() instanceof IStructuredSelection) {
                            IStructuredSelection sel = (IStructuredSelection) event.getSelection();
                            IEntityType definition = (IEntityType) sel.getFirstElement();
                            if (definition == null)
                                clearDescription();
                            else
                                showDescription(definition);
                        }
                    }
                });

        checkboxViewer.addCheckStateListener(new ICheckStateListener() {
            public void checkStateChanged(CheckStateChangedEvent event) {
                checkboxViewer.setSelection(new StructuredSelection(event.getElement()));
            }
        });
    }

    /** 
     * Creates the widgets for the description.
     * @param mainComposite
     */
    private void createDescriptionArea(Composite mainComposite) {

        Font mainFont = mainComposite.getFont();
        Composite textComposite = new Composite(mainComposite, SWT.NONE);
        textComposite.setLayoutData(new GridData(GridData.FILL_BOTH));
        GridLayout textLayout = new GridLayout();
        textLayout.marginWidth = 0;
        textLayout.marginHeight = 0;
        textComposite.setLayout(textLayout);
        textComposite.setFont(mainFont);

        Label descriptionLabel = new Label(textComposite, SWT.NONE);
        descriptionLabel.setText("Description:"); //$NON-NLS-1$
        descriptionLabel.setFont(mainFont);

        descriptionText = new Text(textComposite, SWT.MULTI | SWT.WRAP
                | SWT.READ_ONLY | SWT.BORDER | SWT.H_SCROLL);
        descriptionText.setLayoutData(new GridData(GridData.FILL_BOTH));
        descriptionText.setFont(mainFont);
    }

    /**
     * Populates the list of decorators.
     */
    private void initializeValues() {
        IPreferenceStore store = doGetPreferenceStore();
        
        IEntityType[] definitions = AssociationRuleMiningPlugin.ENTITY_TYPES;
        checkboxViewer.setInput(definitions);
        for (int i = 0; i < definitions.length; i++) {
            boolean isSelected = store.getBoolean(definitions[i].getPreferenceID());
            checkboxViewer.setChecked(definitions[i], isSelected);
        }
        
        boolean supportSelected = store.getBoolean(IRosePreferenceConstants.ROSE_IS_ENABLED_MIN_SUPPORT);
        supportBox.getButton().setSelection(supportSelected);
        int support = store.getInt(IRosePreferenceConstants.ROSE_MIN_SUPPORT);
        int pos = findElement(supportValues, Integer.toString(support));
        if (pos>-1) supportBox.getCombo().select(pos);
        
        boolean confidenceSelected = store.getBoolean(IRosePreferenceConstants.ROSE_IS_ENABLED_MIN_CONFIDENCE);
        confidenceBox.getButton().setSelection(confidenceSelected);
        double confidence = store.getDouble(IRosePreferenceConstants.ROSE_MIN_CONFIDENCE);
        pos = findElement(confidenceValues, Double.toString(confidence));
        if (pos>-1) confidenceBox.getCombo().select(pos);

    }

    /**
     * @param array
     * @param string
     * @return the position, or -1 if not found
     */
    private int findElement(String[] array, String string) {
        for (int i = 0; i < array.length; i++) {
            if (array[i].equals(string)) return i;
        }
        return -1;
    }

    /**
     * Show the selected description in the text.
     * @param definition
     */
    private void showDescription(IEntityType definition) {
        if (descriptionText == null || descriptionText.isDisposed()) {
            return;
        }
        String text = definition.getDescription();
        if (text == null || text.length() == 0)
            descriptionText.setText("No Description."); //$NON-NLS-1$
        else
            descriptionText.setText(text);
    }

    /**
     * Clear the selected description in the text.
     */
    private void clearDescription() {
        if (descriptionText == null || descriptionText.isDisposed()) {
            return;
        }
        descriptionText.setText(""); //$NON-NLS-1$
    }

    /**
     * @see PreferencePage#performDefaults()
     */
    protected void performDefaults() {
        
        IPreferenceStore store = doGetPreferenceStore();
        IEntityType[] definitions = AssociationRuleMiningPlugin.ENTITY_TYPES;
        for (int i = 0; i < definitions.length; i++) {
            store.setToDefault(definitions[i].getPreferenceID());
        }

        store.setToDefault(IRosePreferenceConstants.ROSE_IS_ENABLED_MIN_SUPPORT);
        store.setToDefault(IRosePreferenceConstants.ROSE_IS_ENABLED_MIN_CONFIDENCE);
        store.setToDefault(IRosePreferenceConstants.ROSE_MIN_SUPPORT);
        store.setToDefault(IRosePreferenceConstants.ROSE_MIN_CONFIDENCE);

        super.performDefaults();
        initializeValues();

    }

    /**
     * @see IPreferencePage#performOk()
     */
    public boolean performOk() {
        if (super.performOk()) {
            //Clear the caches first to avoid unneccessary updates
            IPreferenceStore store = doGetPreferenceStore();
            IEntityType[] definitions = AssociationRuleMiningPlugin.ENTITY_TYPES;
            for (int i = 0; i < definitions.length; i++) {
                boolean checked = checkboxViewer.getChecked(definitions[i]);
                definitions[i].setEnabled(checked);
                store.setValue(definitions[i].getPreferenceID(), checked);
            }

            store.setValue(IRosePreferenceConstants.ROSE_IS_ENABLED_MIN_SUPPORT, supportBox.getButton().getSelection());
            store.setValue(IRosePreferenceConstants.ROSE_IS_ENABLED_MIN_CONFIDENCE, confidenceBox.getButton().getSelection());
            
            int support = Integer.parseInt(supportValues[supportBox.getCombo().getSelectionIndex()]);
            store.setValue(IRosePreferenceConstants.ROSE_MIN_SUPPORT, support);

            double confidence = Double.parseDouble(confidenceValues[confidenceBox.getCombo().getSelectionIndex()]);
            store.setValue(IRosePreferenceConstants.ROSE_MIN_CONFIDENCE, confidence);
            
            return true;
        }
        return false;
    }

    protected IPreferenceStore doGetPreferenceStore() {
        return AssistPlugin.getDefault().getPreferenceStore();
    }
    
    /**
     * @see IWorkbenchPreferencePage#init(IWorkbench)
     */
    public void init(IWorkbench workbench) {
        // empty
    }

    private class CheckboxLabelCombo {
        
        private Button button = null;
        private Combo combo = null;
        
        public CheckboxLabelCombo(Composite mainComposite, String description, boolean isEnabled) { 

            Font mainFont = mainComposite.getFont();

            final Composite composite = new Composite(mainComposite, SWT.NONE);
            GridLayout gridLayout = new GridLayout(2, false);
            gridLayout.marginWidth = 0;
            gridLayout.marginHeight = 0;
            composite.setLayout(gridLayout);
            composite.setFont(mainFont);

            button = new Button(composite, SWT.CHECK);
            button.setText(description); 
            button.setFont(mainFont);
            button.setSelection(isEnabled);
            button.addSelectionListener(new SelectionListener() {
                public void widgetSelected(SelectionEvent e) {
                    combo.setEnabled(button.getSelection());
                    composite.redraw();
                 }

                public void widgetDefaultSelected(SelectionEvent e) { //empty
                }
            });
            setButtonLayoutData(button);

            combo = new Combo(composite, SWT.READ_ONLY);
            combo.setFont(mainFont);
            combo.setEnabled(isEnabled);
        }
        
        public Button getButton() {
            return button;
        }
        public Combo getCombo() {
            return combo;
        }
    }
    
}