/*
 * $Id: RoseView.java,v 1.21.2.1 2005/04/15 05:57:00 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.assist.ui.views;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.views.tasklist.ITaskListResourceAdapter;
import org.softevo.rose.RosePluginException;
import org.softevo.rose.assist.AssistPlugin;
import org.softevo.rose.assist.DataMiningResult;
import org.softevo.rose.assist.IChangedEntitiesListener;
import org.softevo.rose.assist.IDataMiningAlgorithm;
import org.softevo.rose.assist.IDataMiningResult;
import org.softevo.rose.compare.ModificationItem;
import org.softevo.rose.db.DatabaseProvider;
import org.softevo.rose.db.DbPlugin;



/** This view presents the results of the data
 *  mining operations as recommondation to the user. <BR>
 *  It is updated whenever the list of changed entities
 *  changed.
 */
public class RoseView extends ViewPart implements IChangedEntitiesListener {
  
  /** The table that presents the results. <BR>
   */
  private TableViewer viewer;
  
  /** The actions in this view. <BR>
   */
  private Vector actions;
  
  /** The set of algorithms available. <BR>
   */
  private Vector /* of MiningAlgorithmDescription */ dataMiningAlgorithms;
  
  /** The algorithm currently active. <BR>
   */
  private MiningAlgorithmDescription activeAlgorithmDescription;
  
  /** The algorithm currently active. <BR>
   */
  private IDataMiningAlgorithm activeAlgorithm = null;
  
  private HashSet focussedProjects = new HashSet();
  private HashMap providerToAlgorithmMap = new HashMap();
  
  private static final String EXT_DMALGORITHM = "org.softevo.rose.assist.DMAlgorithm";
  
  private static final String TAG_ALGORITHM = "algorithm";
  
  private static final String ATT_CLASS = "class";
  private static final String ATT_DESCRIPTION = "description";
  
  private static final int COL_ACTION = 0;
  private static final int COL_ENTITY_NAME = 1;
  private static final int COL_FILE_NAME = 2;
  private static final int COL_SUPPORT = 3;
  private static final int COL_CONFIDENCE = 4;
  
  class ViewContentProvider implements IStructuredContentProvider {
    public void inputChanged(Viewer v, Object oldInput, Object newInput) {
      //System.out.println("Input changed.");
    }
    public void dispose() { // empty
    }
    public Object[] getElements(Object parent) {
      //System.out.println("Get elements for " + parent.getClass());
      if (parent instanceof IDataMiningResult[]) {
        return (IDataMiningResult[]) parent;
      } 
      return null;
    }
  }
  class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {
    public String getColumnText(Object obj, int index) {
      IDataMiningResult result = (IDataMiningResult) obj;
      if (index == COL_CONFIDENCE) {
        String s = Double.toString(result.getConfidence());
        return s.length()>6 ? s.substring(0,6): s;
      } else if (index == COL_SUPPORT) {
        return Integer.toString(result.getSupport());
      } else if (index == COL_ENTITY_NAME) {
        if (result.getEntityName().equals(result.getFileName()))
          return new File(result.getEntityName()).getName();
        return result.getEntityName();
      } else if (index == COL_FILE_NAME) {
        return getFirstAndLast(result.getFileName());
      } else if (index == COL_ACTION) {
          if (result.getAction().equals("DEL_IN")) return "DEL_FROM";
          if (result.getAction().equals("ADD_IN")) return "ADD_TO";
        return result.getAction();
      } else {
        return "";
      }
    }
    public Image getColumnImage(Object obj, int index) {
      return null;
    }
    public Image getImage(Object obj) {
      return null;
    }
    private String getFirstAndLast(String fileName) {
      String[] c = fileName.split("/");
      if (c==null) return fileName;
      int l = c.length;
      if (l<5) {
        return fileName;
      }
      String sep = "/"; // FIXME File.separator;
      return c[0] + sep + c[1] + sep + "..." + sep + c[l-2] + sep + c[l-1];
    }
  }
  class NameSorter extends ViewerSorter {
    private int sortField = 4;
    private int order = 1; // 0 = ascending ; 1 = descending
    public void reverseOrder() {
      order = 1 - order;
    }
    public int getSortby() {
      return sortField;
    }
    public void sortBy(int i) {
      AssistPlugin.debugInfo("Sort by column " + sortField);
      sortField = i;
    }
    public int compare(Viewer v, Object e1, Object e2) {
      String s1 = ((ViewLabelProvider) viewer.getLabelProvider()).getColumnText(e1, sortField);
      String s2 = ((ViewLabelProvider) viewer.getLabelProvider()).getColumnText(e2, sortField);
      if (order == 1) // descending 
        return s2.toLowerCase().compareTo(s1.toLowerCase());
      return s1.toLowerCase().compareTo(s2.toLowerCase()); // else ascending
    }
  }
  
  private ISelectionListener focusListener = new ISelectionListener() {
    public void selectionChanged(IWorkbenchPart part, ISelection selection) {
      RoseView.this.focusSelectionChanged(part, selection);
    }
  };
  
  
  /**
   * The constructor.
   */
  public RoseView() {
    AssistPlugin.getDefault().addChangedEntitiesListener(this);
  }
  
  /** Called whenever the list of changed entities
   *  has changed. <BR>
   *  This may sound strange, but is correct
   *  @param changedEntities the list of changed entities
   */
  public void updateChangedEntities(HashMap changedEntities, Vector changedProjects) {
    // Lazy update
    if (changedProjects==null) {
      recalculate();
      return;          
    }
    for (Iterator iter = changedProjects.iterator(); iter.hasNext();) {
      Object element = iter.next();
      if (focussedProjects.contains(element)) {
        recalculate();
        return;
      }
    }
  }
  
  /** Performs the data mining and updates the view. <BR>
   */
  private void recalculate() {
    ModificationItem[] changedEntities;
    
    if (activeAlgorithmDescription != null) {
      
      // Create new algorithm 
      if (activeAlgorithm == null) {
        // Get database provider
        if (!focussedProjects.isEmpty()) { // there must be at least one (next) IProject
          IProject project = (IProject) focussedProjects.iterator().next(); // FIXME: More database providers are possible
          DatabaseProvider db = DbPlugin.getDefault().getProviderForProject(project); 
          try {
            if (db == null) {
              viewer.setInput(new DataMiningResult[0]);
              viewer.refresh();
            } else if (providerToAlgorithmMap.containsKey(db)) {
              // Use existing algorithm
              activeAlgorithm = (IDataMiningAlgorithm) providerToAlgorithmMap.get(db);
            } else {
              // Create new instance
              Class algorithmClass = Class.forName(activeAlgorithmDescription.getClassName());
              activeAlgorithm = (IDataMiningAlgorithm) algorithmClass.newInstance();
              activeAlgorithm.init(db);
              providerToAlgorithmMap.put(db, activeAlgorithm);
            }
          } catch (InstantiationException e) {
            activeAlgorithm = null;
            AssistPlugin.logException(e, IStatus.WARNING);
          } catch (IllegalAccessException e) {
            activeAlgorithm = null;
            AssistPlugin.logException(e, IStatus.WARNING);
          } catch (ClassNotFoundException e) {
            activeAlgorithm = null;
            AssistPlugin.logException(e, Status.WARNING);
          } catch (RosePluginException e) {
            activeAlgorithm = null;
            AssistPlugin.logException(e, Status.WARNING);
          } finally {
            //empty
          }
        }
      }
      
      // Now there should be a mining algorithm
      if (activeAlgorithm != null) {
        changedEntities = getRelevantModificationItems();
        viewer.setInput(activeAlgorithm.performMining(changedEntities));
        viewer.refresh();
      }
    }
  }
  
  private ModificationItem[] getRelevantModificationItems() {
    Vector union = new Vector();
    for (Iterator iter = focussedProjects.iterator(); iter.hasNext();) {
      IProject project = (IProject) iter.next();
      Vector items = AssistPlugin.getDefault().getChangedEntitiesForProject(project);
      if (items!=null) union.addAll(items);
    }
    return (ModificationItem[]) union.toArray(new ModificationItem[0]);
  }
  
  /**
   * This is a callback that will allow us
   * to create the viewer and initialize it.
   */
  public void createPartControl(Composite parent) {
    
    getSite().getPage().addSelectionListener(focusListener);
    focusSelectionChanged(getSite().getPage().getActivePart(), getSite().getPage().getSelection());
    
    AssistPlugin.getDefault().getPreferenceStore().addPropertyChangeListener(new IPropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent event) {
        for (Iterator iter = providerToAlgorithmMap.values().iterator(); iter.hasNext();) {
          IDataMiningAlgorithm algo = (IDataMiningAlgorithm) iter.next();
          algo.refreshProperties();
        }
        if (activeAlgorithm != null) {
          //activeAlgorithm.refreshProperties();
          recalculate();
        }
      }
    });
    
    viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);
    viewer.getTable().setHeaderVisible(true);
    viewer.getTable().setLinesVisible(true);
    viewer.setContentProvider(new ViewContentProvider());
    viewer.setLabelProvider(new ViewLabelProvider());
    viewer.setSorter(new NameSorter());
    viewer.setInput(new IDataMiningResult[] {});
    
    SelectionListener headerListener = new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        // column selected - need to sort
        NameSorter sorter = (NameSorter) viewer.getSorter();
        int column = viewer.getTable().indexOf((TableColumn) e.widget);
        if (column == sorter.getSortby())
          sorter.reverseOrder();
        else {
          sorter.sortBy(column);
        }
        viewer.refresh();
      }
    };
    //create the table that presents the results
    TableColumn c = new TableColumn(viewer.getTable(), SWT.NONE, 0); 
    c.setText("Action");
    c.setWidth(50);
    c.setResizable(true);
    c.addSelectionListener(headerListener);
    c = new TableColumn(viewer.getTable(), SWT.NONE, 1); 
    c.setText("Symbol");
    c.setWidth(250);
    c.setResizable(true);
    c.addSelectionListener(headerListener);
    c = new TableColumn(viewer.getTable(), SWT.NONE, 2);
    c.setText("File");
    c.setWidth(275);
    c.setResizable(true);
    c.setImage(null);
    c.addSelectionListener(headerListener);
    c = new TableColumn(viewer.getTable(), SWT.RIGHT, 3);
    c.setText("Support");
    c.setWidth(50);
    c.setResizable(true);
    c.setImage(null);
    c.addSelectionListener(headerListener);
    c = new TableColumn(viewer.getTable(), SWT.RIGHT, 4);
    c.setText("Confidence");
    c.setWidth(75);
    c.setResizable(true);
    c.setImage(null);
    c.addSelectionListener(headerListener);
    
    AssistPlugin.debugInfo("Reading data mining algorithms.");
    dataMiningAlgorithms = readDataMiningAlgorithms();
    if (dataMiningAlgorithms.size() > 0) {
      setActiveAlgorithm((MiningAlgorithmDescription) dataMiningAlgorithms.get(0));
    }
    
    AssistPlugin.debugInfo("Reading data mining algorithms (done).");
    recalculate();
    makeActions();
    hookContextMenu();
    hookDoubleClickAction();
    contributeToActionBars();
  }
  
  /**
   * @param activePart
   * @param selection
   */
  private void focusSelectionChanged(IWorkbenchPart activePart, ISelection selection) {
    List resources = new ArrayList();
    if (activePart instanceof IEditorPart) {
      IEditorInput input = ((IEditorPart) activePart).getEditorInput();
      if (input instanceof FileEditorInput) {
        resources.add(((FileEditorInput) input).getFile());
      }
    } else {
      if (selection instanceof IStructuredSelection) {
        for (Iterator iterator = ((IStructuredSelection) selection).iterator(); iterator.hasNext();) {
          Object object = iterator.next();
          if (object instanceof IAdaptable) {
            ITaskListResourceAdapter taskListResourceAdapter;
            Object adapter = ((IAdaptable) object).getAdapter(ITaskListResourceAdapter.class);
            IResource resource = null;
            if (adapter != null && adapter instanceof ITaskListResourceAdapter) {
              taskListResourceAdapter = (ITaskListResourceAdapter) adapter;
              resource = taskListResourceAdapter.getAffectedResource((IAdaptable) object);
            } else {
              IAdaptable adaptable = (IAdaptable) object;
              resource = (IResource) adaptable.getAdapter(IResource.class);
              if (resource == null)
                resource = (IFile) adaptable.getAdapter(IFile.class);
            }
            
            if (resource != null) {
              resources.add(resource);
            }
          }
        }
      }
    }
    
    boolean updateFocussedProjects = false;
    HashSet projects = new HashSet();
    for (Iterator iter = resources.iterator(); iter.hasNext();) {
      IResource resource = (IResource) iter.next();
      projects.add(resource.getProject());
      updateFocussedProjects = updateFocussedProjects || !focussedProjects.contains(resource.getProject());
    }
    
    if (updateFocussedProjects || activeAlgorithm==null) {
      focussedProjects = new HashSet();
      for (Iterator iter = projects.iterator(); iter.hasNext();) {
        IProject project = (IProject) iter.next();
        AssistPlugin.debugInfo("Focus on " + project.getName());
        focussedProjects.addAll(Arrays.asList(DbPlugin.getDefault().getColocatedProjects(project)));
      }
      activeAlgorithm = null; // create new algorithm instance (the old one remains)
      recalculate();
    }    
  }
  
  /** Sets the active data mining algorithm to be used. <BR>
   *  This method also refreshes the view with new results.
   *  @param newAlgorithm the mining algorithm to use
   */
  public void setActiveAlgorithm(MiningAlgorithmDescription newAlgorithm) {
    if (newAlgorithm.equals(activeAlgorithmDescription)) return;
    
    // TODO Close all running algorithms (Is this necessary?)
    /*
     for (Iterator iter = providerToAlgorithmMap.values().iterator(); iter.hasNext();) {
     IDataMiningAlgorithm runningAlgorithmMap = (IDataMiningAlgorithm) iter.next();
     try {
     runningAlgorithmMap.dispose();
     } catch (RosePluginException e) {
     System.out.println("Failed to dispose algorithm " + runningAlgorithmMap.getName());
     e.printStackTrace();
     }
     }
     */
    
    // Reset algorithm map
    providerToAlgorithmMap = new HashMap();
    
    // Update active algorithms
    activeAlgorithm = null;
    activeAlgorithmDescription = newAlgorithm;
    
    // Create algorithm for current selection & refresh view
    recalculate();
  }
  
  private Vector readDataMiningAlgorithms() {
    
    Vector result = new Vector();
    IConfigurationElement[] elements = Platform.getExtensionRegistry().getConfigurationElementsFor(EXT_DMALGORITHM);
    for (int j = 0; j < elements.length; j++) {
      if (elements[j].getName().toLowerCase().equals(TAG_ALGORITHM)
          && elements[j].getAttribute(ATT_CLASS) != null
          && elements[j].getAttribute(ATT_DESCRIPTION) != null) {
        String className = elements[j].getAttribute(ATT_CLASS);
        String description = elements[j].getAttribute(ATT_DESCRIPTION);
        MiningAlgorithmDescription algorithmDescription = 
          new MiningAlgorithmDescription(className, description);
        result.add(algorithmDescription);
      }
    }
    return result;
  }
  
  
  private void hookContextMenu() {
    MenuManager menuMgr = new MenuManager("#PopupMenu");
    menuMgr.setRemoveAllWhenShown(true);
    menuMgr.addMenuListener(new IMenuListener() {
      public void menuAboutToShow(IMenuManager manager) {
        RoseView.this.fillContextMenu(manager);
      }
    });
    Menu menu = menuMgr.createContextMenu(viewer.getControl());
    viewer.getControl().setMenu(menu);
    getSite().registerContextMenu(menuMgr, viewer);
  }
  
  private void contributeToActionBars() {
    IActionBars bars = getViewSite().getActionBars();
    fillLocalPullDown(bars.getMenuManager());
    fillLocalToolBar(bars.getToolBarManager());
  }
  
  private void fillLocalPullDown(IMenuManager manager) {
    Action action;
    
    for (int counter = 0; counter < actions.size(); counter++) {
      action = (Action) actions.get(counter);
      manager.add(action);
    }
  }
  
  private void fillContextMenu(IMenuManager manager) {
    //manager.add(action1);
    //manager.add(action2);
    // Other plug-ins can contribute there actions here
    manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
  }
  
  private void fillLocalToolBar(IToolBarManager manager) {
    //IDataMiningAlgorithm algorithm;
    
    //for (int counter = 0; counter < dataMiningAlgorithms.size(); counter++) {
    //algorithm = (IDataMiningAlgorithm) dataMiningAlgorithms.get(counter);
    // FIXME ???
    //}
  }
  
  private void makeActions() {
    actions = new Vector();
    
    // Create set mining actions
    for (int counter = 0; counter < dataMiningAlgorithms.size(); counter++) {
      MiningAlgorithmDescription desc = (MiningAlgorithmDescription) dataMiningAlgorithms.get(counter);
      actions.add(new SetMiningAlgorithmAction(desc));
    }
    
  }
  
  
  private void openSelection(ISelection selection) {
    if (selection instanceof IStructuredSelection) {
      Object o = ((IStructuredSelection) selection).getFirstElement();
      if (o instanceof IDataMiningResult) {
        IDataMiningResult item = (IDataMiningResult) o;
        openItem(item);
      }
    }
  }
  
  private void openItem(IDataMiningResult item) {
    IProject[] projects = new IProject[focussedProjects.size()];
    int i = 0;
    for (Iterator iter = focussedProjects.iterator(); iter.hasNext(); i++) {
      projects[i] = (IProject) iter.next();
    }
    AssistPlugin.getDefault().openInEditor(item.getFileName(), item.getEntityName(), projects);
  }
  
  
  private class SetMiningAlgorithmAction extends Action {
    
    private MiningAlgorithmDescription miningDescription;
    
    public SetMiningAlgorithmAction(MiningAlgorithmDescription miningDescription) {
      super("Use " + miningDescription.getDescription());
      this.miningDescription = miningDescription;
    }
    
    public void run() {
      setActiveAlgorithm(miningDescription);
      //recalculate();
    }
  }
  
  private void hookDoubleClickAction() {
    viewer.addDoubleClickListener(new IDoubleClickListener() {
      public void doubleClick(DoubleClickEvent event) {
        openSelection(event.getSelection());
      }
    });
  }
  
  /**
   * Passing the focus request to the viewer's control.
   */
  public void setFocus() {
    viewer.getControl().setFocus();
  }
  
  private class MiningAlgorithmDescription {
    private String className = null;
    private String description = null;
    public MiningAlgorithmDescription(String className, String description) {
      if (className==null) throw new NullPointerException("classname must not be null");
      this.className = className;
      this.description = description;
    }
    
    public boolean equals(Object obj) {
      if (obj instanceof MiningAlgorithmDescription) 
        return className.equals(((MiningAlgorithmDescription) obj).getClassName());
      return super.equals(obj);
    }    
    public int hashCode() {
      return className.hashCode();
    }
    public String getClassName() {
      return className;
    }
    public String getDescription() {
      return description;
    }
  }
}