/*
 * $Id: RoseComparePlugin.java,v 1.10 2005/03/01 12:55:26 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.compare;

import java.util.LinkedList;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.compare.ResourceNode;
import org.eclipse.compare.internal.CompareUIPlugin;
import org.eclipse.compare.internal.StructureCreatorDescriptor;
import org.eclipse.compare.structuremergeviewer.DiffContainer;
import org.eclipse.compare.structuremergeviewer.DiffNode;
import org.eclipse.compare.structuremergeviewer.Differencer;
import org.eclipse.compare.structuremergeviewer.IStructureComparator;
import org.eclipse.compare.structuremergeviewer.IStructureCreator;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.Status;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
public class RoseComparePlugin extends Plugin {
	//The shared instance.
	private static RoseComparePlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;
	
	private static final String ID = "org.softevo.rose.compare";

	
	/**
	 * The constructor.
	 */
	public RoseComparePlugin() {
		super();
		plugin = this;
		try {
			resourceBundle = ResourceBundle.getBundle("org.softevo.rose.compare.ComparePluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
	}

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
	}

	/**
	 * Returns the shared instance.
	 */
	public static RoseComparePlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = RoseComparePlugin.getDefault().getResourceBundle();
		try {
			return (bundle != null) ? bundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}
		
	/**
	 * Returns an structure creator descriptor for the given type.
	 *
	 * @param type the type for which to find a descriptor
	 * @return a descriptor for the given type, or <code>null</code> if no
	 *   descriptor has been registered
	 */
	public static IStructureCreator getStructureCreator(String type) {
	    if (type == null) return null;
	    if (type.startsWith(".")) type = type.substring(1);
	    StructureCreatorDescriptor creator = CompareUIPlugin.getDefault().getStructureCreator(type);
	    if (creator == null) return null;
	    return creator.createStructureCreator();
	}

	
	/**
	 * 
	 * @param leftFile
	 * @param rightFile
	 * @return
	 */
	public static LinkedList doStructureCompare(IStructureCreator creator, IFile leftFile, IFile rightFile) {
 	    return doStructureCompare(creator, new ResourceNode(leftFile), new ResourceNode(rightFile));
	}

	
	/**
	 * Left and right objects "should" implement IStreamContentAccessor.
	 * E.g.:
	 * <ul>
	 * <li><code>new ResourceNode(leftFile)</code>
	 * <li><code>new HistoryItem(oldFile, IFileState)</code>
	 * </ul>
	 * @param creator
	 * @param left
	 * @param right
	 * @return
	 */
	public static LinkedList doStructureCompare(IStructureCreator creator, Object left, Object right) {
 		// Create the structure of the two versions
		IStructureComparator leftStructure = creator.getStructure(left);
		IStructureComparator rightStructure = creator.getStructure(right);
		
 		return doStructureCompare(leftStructure, rightStructure);
	}

	
    /**
     * @param leftStructure
     * @param rightStructure
     * @return
     */
    public static LinkedList doStructureCompare(IStructureComparator leftStructure, IStructureComparator rightStructure) {
        LinkedList result = new LinkedList();

 		// Create the differencer
		Differencer differencer = new Differencer();

		// Find the differences
		// System.out.println("Create differences by findDifferences...");
		Object outcome =
			differencer.findDifferences(
				false,
				new NullProgressMonitor(),
				null,
				null,
				leftStructure,
				rightStructure);		
		// System.out.println(outcome);

		// Analyze outcome via a DFS
		if (outcome != null && outcome instanceof DiffNode) {
			DiffNode dn = (DiffNode) outcome;
					
			// Initialize DFS stack
			LinkedList stack = new LinkedList();
			for (int i = 0; i < dn.getChildren().length; i++) {
				stack.addLast(dn.getChildren()[i]);
			}

			// Work off stack
			while (!stack.isEmpty()) {
				Object element = stack.removeFirst();

				if (element instanceof DiffContainer) {
					DiffContainer dc = (DiffContainer) element;

					if (dc.getChildren() == null 
						|| dc.getChildren().length==0) {
						// We have reached a leaf - found a difference
						if (dc instanceof DiffNode) {
							// Add difference to list
							result.addAll(getModificationItems((DiffNode) dc));
						}
					} else {
						// Push elements on stack (in reverse order)
						for (int i = dc.getChildren().length-1 ; i > -1; i--) {
							stack.addFirst(dc.getChildren()[i]);
						}									
					}
				}
			}
		}
		
		//for (Iterator iter = result.iterator(); iter.hasNext();) {
         //   ModificationItem element = (ModificationItem) iter.next();
         //   if (element != null && element.isDocumentRangeNode()) {
         //       DocumentRangeNode drn = element.getDocumentRangeNode();
         //       System.out.println(drn.getId());                
         //   }
         //}
        return result;
    }

    /**
     * @param dc
     * @return
     */
    private static LinkedList getModificationItems(DiffNode dn) {

		int type = ModificationItem.CHANGED; 
		if (dn.getLeft() == null) {
			type = ModificationItem.ADDED; 
		} else if (dn.getRight() == null) {
			type = ModificationItem.DELETED;
		}
		
		return ModificationItem.createModificationItems(dn.getId(), type);   
    }

	public static void debugInfo(String message) {
	    if (plugin.isDebugging())
	        System.out.println(message);
	}

	public static void logInfo(String message) {
	    if (plugin.isDebugging())
	        plugin.getLog().log(new Status(IStatus.INFO, ID, 0, message, null));
	}

	public static void logWarning(String message, Throwable e) {
		plugin.getLog().log(new Status(IStatus.WARNING, ID, 0, message, e));
	}

	public static void logError(String message, Throwable e) {
		plugin.getLog().log(new Status(IStatus.ERROR, ID, 0, message, e));
	}
	
	public static void logException(Throwable e, int severity) {
	    plugin.getLog().log(new Status(severity, ID, 0, e.getMessage(), e));
	} 

}
