/*
 * $Id: InternalDBConnectionProperties.java,v 1.5 2005/03/01 12:54:52 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.db;

import java.io.File;
import java.util.Properties;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IPath;
import org.softevo.rose.db.jdbc.JdbcPlugin;

/** Special properties object that represents the
 *  internal database connection. <BR>
 *  The database url depends on the project the
 *  properties object is instantiated for.
 */
public class InternalDBConnectionProperties extends DBConnectionProperties {

    /**
     * 
     */
    private static final long serialVersionUID = 3257005453866382137L;

/** String constant for name. <BR>
   */
  public static final String NAME_INTERNALDB = "Internal Database";

  /** The directory where database data is stored. <BR>
   */
  public static final String DIR_DBDATA = "rose";

  /** The name of the database. <BR>
   */
  public static final String NAME_DB = "rosedb";

  /** Creates a new internal properties object and intializes
   *  with values appropriate for the project passed. <BR>
   *  @param project the project the properties object is instantiated for
   */
  public InternalDBConnectionProperties(IProject project) {
    super(NAME_INTERNALDB, getDatabaseURL(project), "sa", "", null);
    databaseProperties = JdbcPlugin.getDefault().getProperties(JdbcPlugin.NAME_INTERNALDRIVER);
  }

  /** Creates a fake properties object that
   *  may be used for display purposes in the tree view. <BR>
   *  This constructor does not init the url.
   */
  public InternalDBConnectionProperties() {
    super(NAME_INTERNALDB, "", "sa", "", null);
    databaseProperties = JdbcPlugin.getDefault().getProperties(JdbcPlugin.NAME_INTERNALDRIVER);
  }
  
  /** This properties object is not editable. <BR>
   */
  public boolean isEditable() {
    return false;
  }

  /** Returns a copy with the same values as this 
   *  object. <BR>
   */
  public Object clone() {
    InternalDBConnectionProperties result;
    result =  new InternalDBConnectionProperties();
    result.databaseURL = databaseURL;
    result.name = name;
    result.password = password;
    result.userName = userName;
    return result;
  }  
  /** Sets the properties from this object in
   *  the java properties object. <BR>
   *  @param the java properties object
   */
  public void setProperties(Properties properties) {
    properties.setProperty(KEY_NAME, name);
  }

  /** Creates the database url for the hsql database. <BR>
   *  @param project the project the properties are created for
   *  @return the database url
   */
  private static String getDatabaseURL(IProject project) {
    IPath workingPath;
    
    workingPath = project.getWorkingLocation(DbPlugin.ID);
    if (workingPath != null) {
      return "jdbc:hsqldb:file:" + workingPath.toFile().getAbsolutePath() + 
             File.separator + DIR_DBDATA + File.separator + NAME_DB;
    }
    //TODO find a proper return value
    return null;
  }
}
